@@ -30,13 +30,18 @@ enum {
     FILES_CHANGED,
     LOADED,
     UNMOUNT,
+    CHANGED,
+    REMOVED,
+    CONTENT_CHANGED,
+    FS_INFO,
     ERROR,
     N_SIGNALS
 };
 
 static FmFolder* fm_folder_new_internal(FmPath* path, GFile* gf);
 static FmFolder* fm_folder_get_internal(FmPath* path, GFile* gf);
-static void fm_folder_finalize              (GObject *object);
+static void fm_folder_finalize(GObject *object);
+static void fm_folder_content_changed(FmFolder* folder);
 
 static void on_file_info_finished(FmFileInfoJob* job, FmFolder* folder);
 static gboolean on_idle(FmFolder* folder);
@@ -51,10 +56,14 @@ static GHashTable* hash = NULL; /* FIXME: should this be guarded with a mutex? *
 static void fm_folder_class_init(FmFolderClass *klass)
 {
     GObjectClass *g_object_class;
+    FmFolderClass* folder_class;
     g_object_class = G_OBJECT_CLASS(klass);
     g_object_class->finalize = fm_folder_finalize;
     fm_folder_parent_class = (GObjectClass*)g_type_class_peek(G_TYPE_OBJECT);
 
+    folder_class = FM_FOLDER_CLASS(klass);
+    folder_class->content_changed = fm_folder_content_changed;
+
     /*
     * files-added is emitted when there is a new file created in the dir.
     * The param is GList* of the newly added file.
@@ -103,6 +112,7 @@ static void fm_folder_class_init(FmFolderClass *klass)
                        g_cclosure_marshal_VOID__VOID,
                        G_TYPE_NONE, 0);
 
+    /* emitted when the folder is unmounted */
     signals[ UNMOUNT ] =
         g_signal_new ( "unmount",
                        G_TYPE_FROM_CLASS ( klass ),
@@ -112,6 +122,47 @@ static void fm_folder_class_init(FmFolderClass *klass)
                        g_cclosure_marshal_VOID__VOID,
                        G_TYPE_NONE, 0);
 
+    /* emitted when the folder itself is changed */
+    signals[ CHANGED ] =
+        g_signal_new ( "changed",
+                       G_TYPE_FROM_CLASS ( klass ),
+                       G_SIGNAL_RUN_FIRST,
+                       G_STRUCT_OFFSET ( FmFolderClass, changed ),
+                       NULL, NULL,
+                       g_cclosure_marshal_VOID__VOID,
+                       G_TYPE_NONE, 0);
+
+    /* emitted when the folder itself is deleted */
+    signals[ REMOVED ] =
+        g_signal_new ( "removed",
+                       G_TYPE_FROM_CLASS ( klass ),
+                       G_SIGNAL_RUN_FIRST,
+                       G_STRUCT_OFFSET ( FmFolderClass, removed ),
+                       NULL, NULL,
+                       g_cclosure_marshal_VOID__VOID,
+                       G_TYPE_NONE, 0);
+
+    /* emitted when content of the folder is changed (some files are added, removed, changed) */
+    signals[ CONTENT_CHANGED ] =
+        g_signal_new ( "content-changed",
+                       G_TYPE_FROM_CLASS ( klass ),
+                       G_SIGNAL_RUN_FIRST,
+                       G_STRUCT_OFFSET ( FmFolderClass, content_changed ),
+                       NULL, NULL,
+                       g_cclosure_marshal_VOID__VOID,
+                       G_TYPE_NONE, 0);
+
+    /* emitted when filesystem information is available. */
+    signals[ FS_INFO ] =
+        g_signal_new ( "fs-info",
+                       G_TYPE_FROM_CLASS ( klass ),
+                       G_SIGNAL_RUN_FIRST,
+                       G_STRUCT_OFFSET ( FmFolderClass, fs_info ),
+                       NULL, NULL,
+                       g_cclosure_marshal_VOID__VOID,
+                       G_TYPE_NONE, 0);
+
+    /* emitted when errors happen */
     signals[ERROR] =
         g_signal_new( "error",
                       G_TYPE_FROM_CLASS ( klass ),
@@ -123,9 +174,9 @@ static void fm_folder_class_init(FmFolderClass *klass)
 }
 
 
-static void fm_folder_init(FmFolder *self)
+static void fm_folder_init(FmFolder *folder)
 {
-    self->files = fm_file_info_list_new();
+    folder->files = fm_file_info_list_new();
 }
 
 void on_file_info_finished(FmFileInfoJob* job, FmFolder* folder)
@@ -133,6 +184,7 @@ void on_file_info_finished(FmFileInfoJob* job, FmFolder* folder)
     GList* l;
     GSList* files_to_add = NULL;
     GSList* files_to_update = NULL;
+    gboolean content_changed = FALSE;
 
     for(l=fm_list_peek_head_link(job->file_infos);l;l=l->next)
     {
@@ -161,13 +213,18 @@ void on_file_info_finished(FmFileInfoJob* job, FmFolder* folder)
     {
         g_signal_emit(folder, signals[FILES_ADDED], 0, files_to_add);
         g_slist_free(files_to_add);
+        content_changed = TRUE;
     }
     if(files_to_update)
     {
         g_signal_emit(folder, signals[FILES_CHANGED], 0, files_to_update);
         g_slist_free(files_to_update);
+        content_changed = TRUE;
     }
 
+    if(content_changed)
+        g_signal_emit(folder, signals[CONTENT_CHANGED], 0);
+
     folder->pending_jobs = g_slist_remove(folder->pending_jobs, job);
 }
 
@@ -243,6 +300,8 @@ gboolean on_idle(FmFolder* folder)
         g_slist_foreach(folder->files_to_del, (GFunc)fm_file_info_unref, NULL);
         g_slist_free(folder->files_to_del);
         folder->files_to_del = NULL;
+
+        g_signal_emit(folder, signals[CONTENT_CHANGED], 0);
     }
 
     return FALSE;
@@ -271,14 +330,26 @@ static void on_folder_changed(GFileMonitor* mon, GFile* gf, GFile* other, GFileM
     {
         g_debug("event of the folder itself: %d", evt);
         /* FIXME: handle unmount events */
-        if(evt == G_FILE_MONITOR_EVENT_PRE_UNMOUNT)
+        switch(evt)
         {
+        case G_FILE_MONITOR_EVENT_PRE_UNMOUNT:
             g_debug("folder is going to be unmounted");
-        }
-        else if(evt == G_FILE_MONITOR_EVENT_UNMOUNTED)
-        {
+            break;
+        case G_FILE_MONITOR_EVENT_UNMOUNTED:
             g_signal_emit(folder, signals[UNMOUNT], 0);
             g_debug("folder is unmounted");
+            break;
+        case G_FILE_MONITOR_EVENT_DELETED:
+            g_signal_emit(folder, signals[REMOVED], 0);
+            g_debug("folder is deleted");
+            break;
+        case G_FILE_MONITOR_EVENT_ATTRIBUTE_CHANGED:
+        case G_FILE_MONITOR_EVENT_CHANGES_DONE_HINT:
+            g_signal_emit(folder, signals[CHANGED], 0);
+            /* update volume info */
+            fm_folder_query_filesystem_info(folder);
+            g_debug("folder is changed");
+            break;
         }
         return;
     }
@@ -334,8 +405,8 @@ static void on_job_finished(FmDirListJob* job, FmFolder* folder)
 
 static FmJobErrorAction on_job_err(FmDirListJob* job, GError* err, FmJobErrorSeverity severity, FmFolder* folder)
 {
-	FmJobErrorAction ret;
-	g_signal_emit(folder, signals[ERROR], 0, err, severity, &ret);
+    FmJobErrorAction ret;
+    g_signal_emit(folder, signals[ERROR], 0, err, severity, &ret);
     return ret;
 }
 
@@ -389,67 +460,73 @@ FmFolder* fm_folder_get_internal(FmPath* path, GFile* gf)
 
 static void fm_folder_finalize(GObject *object)
 {
-    FmFolder *self;
+    FmFolder *folder;
     g_return_if_fail(object != NULL);
     g_return_if_fail(FM_IS_FOLDER(object));
 
-    self = FM_FOLDER(object);
+    folder = FM_FOLDER(object);
 
-    if(self->job)
+    if(folder->job)
     {
-        g_signal_handlers_disconnect_by_func(self->job, on_job_finished, self);
-        g_signal_handlers_disconnect_by_func(self->job, on_job_err, self);
-        fm_job_cancel(FM_JOB(self->job)); /* FIXME: is this ok? */
+        g_signal_handlers_disconnect_by_func(folder->job, on_job_finished, folder);
+        g_signal_handlers_disconnect_by_func(folder->job, on_job_err, folder);
+        fm_job_cancel(FM_JOB(folder->job)); /* FIXME: is this ok? */
         /* the job will be freed automatically in idle handler. */
     }
 
-    if(self->pending_jobs)
+    if(folder->pending_jobs)
     {
         GSList* l;
-        for(l = self->pending_jobs;l;l=l->next)
+        for(l = folder->pending_jobs;l;l=l->next)
         {
             FmJob* job = FM_JOB(l->data);
-            g_signal_handlers_disconnect_by_func(job, on_job_finished, self);
+            g_signal_handlers_disconnect_by_func(job, on_job_finished, folder);
             fm_job_cancel(job);
             /* the job will be freed automatically in idle handler. */
         }
     }
 
     /* remove from hash table */
-    g_hash_table_remove(hash, self->dir_path);
-    if(self->dir_path)
-        fm_path_unref(self->dir_path);
+    g_hash_table_remove(hash, folder->dir_path);
+    if(folder->dir_path)
+        fm_path_unref(folder->dir_path);
 
-    if(self->dir_fi)
-        fm_file_info_unref(self->dir_fi);
+    if(folder->dir_fi)
+        fm_file_info_unref(folder->dir_fi);
 
-    if(self->gf)
-        g_object_unref(self->gf);
+    if(folder->gf)
+        g_object_unref(folder->gf);
 
-    if(self->mon)
+    if(folder->mon)
     {
-        g_signal_handlers_disconnect_by_func(self->mon, on_folder_changed, self);
-        g_object_unref(self->mon);
+        g_signal_handlers_disconnect_by_func(folder->mon, on_folder_changed, folder);
+        g_object_unref(folder->mon);
     }
 
-    if(self->idle_handler)
+    if(folder->idle_handler)
     {
-        g_source_remove(self->idle_handler);
-        if(self->files_to_add)
+        g_source_remove(folder->idle_handler);
+        if(folder->files_to_add)
         {
-            g_slist_foreach(self->files_to_add, (GFunc)g_free, NULL);
-            g_slist_free(self->files_to_add);
+            g_slist_foreach(folder->files_to_add, (GFunc)g_free, NULL);
+            g_slist_free(folder->files_to_add);
         }
-        if(self->files_to_update)
+        if(folder->files_to_update)
         {
-            g_slist_foreach(self->files_to_update, (GFunc)g_free, NULL);
-            g_slist_free(self->files_to_update);
+            g_slist_foreach(folder->files_to_update, (GFunc)g_free, NULL);
+            g_slist_free(folder->files_to_update);
         }
-        if(self->files_to_del)
-            g_slist_free(self->files_to_del);
+        if(folder->files_to_del)
+            g_slist_free(folder->files_to_del);
     }
 
-    fm_list_unref(self->files);
+    fm_list_unref(folder->files);
+
+    if(folder->fs_size_cancellable)
+    {
+        g_cancellable_cancel(folder->fs_size_cancellable);
+        g_object_unref(folder->fs_size_cancellable);
+    }
 
     if (G_OBJECT_CLASS(fm_folder_parent_class)->finalize)
         (* G_OBJECT_CLASS(fm_folder_parent_class)->finalize)(object);
@@ -463,14 +540,9 @@ FmFolder*    fm_folder_get_for_gfile    (GFile* gf)
     return folder;
 }
 
-FmFolder* fm_folder_get_for_path(FmPath* path)
-{
-    return fm_folder_get_internal(path, NULL);
-}
-
 FmFolder* fm_folder_get_for_path_name(const char* path)
 {
-    FmPath* fm_path = fm_path_new(path);
+    FmPath* fm_path = fm_path_new_for_str(path);
     FmFolder* folder = fm_folder_get_internal(fm_path, NULL);
     fm_path_unref(fm_path);
     return folder;
@@ -500,6 +572,8 @@ void fm_folder_reload(FmFolder* folder)
         g_signal_emit(folder, signals[FILES_REMOVED], 0, files_to_del);
         fm_list_clear(folder->files); /* fm_file_info_unref will be invoked. */
         g_slist_free(files_to_del);
+
+        g_signal_emit(folder, signals[CONTENT_CHANGED], 0);
     }
 
     folder->job = fm_dir_list_job_new(folder->dir_path, FALSE);
@@ -536,7 +610,78 @@ FmFolder* fm_folder_get(FmPath* path)
     return fm_folder_get_internal(path, NULL);
 }
 
-gboolean fm_folder_get_is_loading(FmFolder* folder)
+gboolean fm_folder_get_is_loaded(FmFolder* folder)
 {
-    return (folder->job != NULL);
+    return (folder->job == NULL);
+}
+
+gboolean fm_folder_get_filesystem_info(FmFolder* folder, guint64* total_size, guint64* free_size)
+{
+    if(folder->has_fs_info)
+    {
+        *total_size = folder->fs_total_size;
+        *free_size = folder->fs_free_size;
+        return TRUE;
+    }
+    return FALSE;
+}
+
+static void on_query_filesystem_info_finished(GObject *src, GAsyncResult *res, FmFolder* folder)
+{
+    GFile* gf = G_FILE(src);
+    GError* err = NULL;
+    GFileInfo* inf = g_file_query_filesystem_info_finish(gf, res, &err);
+    if(!inf)
+    {
+        folder->fs_total_size = folder->fs_free_size = 0;
+        folder->has_fs_info = FALSE;
+        folder->fs_info_not_avail = TRUE;
+
+        /* FIXME: examine unsupported filesystems */
+
+        g_error_free(err);
+        goto _out;
+    }
+    if(g_file_info_has_attribute(inf, G_FILE_ATTRIBUTE_FILESYSTEM_SIZE))
+    {
+        folder->fs_total_size = g_file_info_get_attribute_uint64(inf, G_FILE_ATTRIBUTE_FILESYSTEM_SIZE);
+        folder->fs_free_size = g_file_info_get_attribute_uint64(inf, G_FILE_ATTRIBUTE_FILESYSTEM_FREE);
+        folder->has_fs_info = TRUE;
+    }
+    else
+    {
+        folder->fs_total_size = folder->fs_free_size = 0;
+        folder->has_fs_info = FALSE;
+        folder->fs_info_not_avail = TRUE;
+    }
+    g_object_unref(inf);
+
+_out:
+    if(folder->fs_size_cancellable)
+    {
+        g_object_unref(folder->fs_size_cancellable);
+        folder->fs_size_cancellable = NULL;
+    }
+
+    g_signal_emit(folder, signals[FS_INFO], 0);
+    g_object_unref(folder);
+}
+
+void fm_folder_query_filesystem_info(FmFolder* folder)
+{
+    if(!folder->fs_size_cancellable && !folder->fs_info_not_avail)
+    {
+        folder->fs_size_cancellable = g_cancellable_new();
+        g_file_query_filesystem_info_async(folder->gf,
+                G_FILE_ATTRIBUTE_FILESYSTEM_SIZE","
+                G_FILE_ATTRIBUTE_FILESYSTEM_FREE,
+                G_PRIORITY_LOW, folder->fs_size_cancellable,
+                on_query_filesystem_info_finished, g_object_ref(folder));
+    }
+}
+
+static void fm_folder_content_changed(FmFolder* folder)
+{
+    if(folder->has_fs_info && !folder->fs_info_not_avail)
+        fm_folder_query_filesystem_info(folder);
 }
