@@ -33,30 +33,30 @@ G_BEGIN_DECLS
 #define FM_PATH(path)   ((FmPath*)path)
 
 typedef struct _FmPath FmPath;
-typedef enum _FmPathFlags FmPathFlags;
 typedef FmList FmPathList;
 
 enum _FmPathFlags
 {
-	FM_PATH_NONE = 0,
-	FM_PATH_IS_NATIVE = 1<<0, /* This is a native path to UNIX, like /home */
-	FM_PATH_IS_LOCAL = 1<<1, /* This path refers  to a file on local filesystem */
-	FM_PATH_IS_VIRTUAL = 1<<2, /* This path is virtual and it doesn't exist on real filesystem */
-	FM_PATH_IS_TRASH = 1<<3, /* This path is under trash:/// */
-	FM_PATH_IS_XDG_MENU = 1<<4, /* This path is under menu:/// */
+    FM_PATH_NONE = 0,
+    FM_PATH_IS_NATIVE = 1<<0, /* This is a native path to UNIX, like /home */
+    FM_PATH_IS_LOCAL = 1<<1, /* This path refers  to a file on local filesystem */
+    FM_PATH_IS_VIRTUAL = 1<<2, /* This path is virtual and it doesn't exist on real filesystem */
+    FM_PATH_IS_TRASH = 1<<3, /* This path is under trash:/// */
+    FM_PATH_IS_XDG_MENU = 1<<4, /* This path is under menu:/// */
 
-	/* reserved for future use */
-	FM_PATH_IS_RESERVED1 = 1<<5,
-	FM_PATH_IS_RESERVED2 = 1<<6,
-	FM_PATH_IS_RESERVED3 = 1<<7,
+    /* reserved for future use */
+    FM_PATH_IS_RESERVED1 = 1<<5,
+    FM_PATH_IS_RESERVED2 = 1<<6,
+    FM_PATH_IS_RESERVED3 = 1<<7,
 };
+typedef enum _FmPathFlags FmPathFlags;
 
 struct _FmPath
 {
-	gint n_ref;
-	FmPath* parent;
-	guchar flags; /* FmPathFlags flags : 8; */
-	char name[1];
+    gint n_ref;
+    FmPath* parent;
+    guchar flags; /* FmPathFlags flags : 8; */
+    char name[1];
 };
 
 void _fm_path_init();
@@ -82,7 +82,7 @@ FmPath* fm_path_get_desktop(); /* $HOME/Desktop */
 FmPath* fm_path_get_trash(); /* trash:/// */
 FmPath* fm_path_get_apps_menu(); /* menu://applications.menu/ */
 
-FmPath*	fm_path_ref(FmPath* path);
+FmPath* fm_path_ref(FmPath* path);
 void fm_path_unref(FmPath* path);
 
 FmPath* fm_path_get_parent(FmPath* path);
@@ -111,6 +111,9 @@ gboolean fm_path_equal(FmPath* p1, FmPath* p2);
 /* used for completion in fm_path_entry */
 gboolean fm_path_equal_str(FmPath *path, const gchar *str, int n);
 
+/* calculate how many elements are in this path. */
+int fm_path_depth(FmPath* path);
+
 /* path list */
 FmPathList* fm_path_list_new();
 FmPathList* fm_path_list_new_from_uri_list(const char* uri_list);
