@@ -37,23 +37,23 @@ G_BEGIN_DECLS
 /* Convinient dialog functions */
 
 /* Display an error message to the user */
-void fm_show_error(GtkWindow* parent, const char* msg);
+void fm_show_error(GtkWindow* parent, const char* title, const char* msg);
 
 /* Ask the user a yes-no question. */
-gboolean fm_yes_no(GtkWindow* parent, const char* question, gboolean default_yes);
-gboolean fm_ok_cancel(GtkWindow* parent, const char* question, gboolean default_ok);
+gboolean fm_yes_no(GtkWindow* parent, const char* title, const char* question, gboolean default_yes);
+gboolean fm_ok_cancel(GtkWindow* parent, const char* title, const char* question, gboolean default_ok);
 
 /* Ask the user a question with a NULL-terminated array of
  * options provided. The return value was index of the selected option. */
-int fm_ask(GtkWindow* parent, const char* question, ...);
-int fm_askv(GtkWindow* parent, const char* question, const char** options);
-int fm_ask_valist(GtkWindow* parent, const char* question, va_list options);
+int fm_ask(GtkWindow* parent, const char* title, const char* question, ...);
+int fm_askv(GtkWindow* parent, const char* title, const char* question, const char** options);
+int fm_ask_valist(GtkWindow* parent, const char* title, const char* question, va_list options);
 
 char* fm_get_user_input(GtkWindow* parent, const char* title, const char* msg, const char* default_text);
 FmPath* fm_get_user_input_path(GtkWindow* parent, const char* title, const char* msg, FmPath* default_path);
 
 /* Ask the user to select a folder. */
-FmPath* fm_select_folder(GtkWindow* parent);
+FmPath* fm_select_folder(GtkWindow* parent, const char* title);
 
 /* Mount */
 gboolean fm_mount_path(GtkWindow* parent, FmPath* path, gboolean interactive);
@@ -64,46 +64,40 @@ gboolean fm_eject_mount(GtkWindow* parent, GMount* mount, gboolean interactive);
 gboolean fm_eject_volume(GtkWindow* parent, GVolume* vol, gboolean interactive);
 
 /* File operations */
-void fm_copy_files(FmPathList* files, FmPath* dest_dir);
-void fm_move_files(FmPathList* files, FmPath* dest_dir);
+void fm_copy_files(GtkWindow* parent, FmPathList* files, FmPath* dest_dir);
+void fm_move_files(GtkWindow* parent, FmPathList* files, FmPath* dest_dir);
 
-#define fm_copy_file(file, dest_dir) \
+#define fm_copy_file(parent, file, dest_dir) \
     G_STMT_START {    \
         FmPathList* files = fm_path_list_new(); \
         fm_list_push_tail(files, file); \
-        fm_copy_files(files, dest_dir); \
+        fm_copy_files(parent, files, dest_dir); \
         fm_list_unref(files);   \
     } G_STMT_END
 
-#define fm_move_file(file, dest_dir) \
+#define fm_move_file(parent, file, dest_dir) \
     G_STMT_START {    \
     FmPathList* files = fm_path_list_new(); \
     fm_list_push_tail(files, file); \
-    fm_move_files(files, dest_dir); \
+    fm_move_files(parent, files, dest_dir); \
     fm_list_unref(files);   \
     } G_STMT_END
 
-void fm_move_or_copy_files_to(FmPathList* files, gboolean is_move);
-#define fm_move_files_to(files)   fm_move_or_copy_files_to(files, TRUE)
-#define fm_copy_files_to(files)   fm_move_or_copy_files_to(files, FALSE)
+void fm_move_or_copy_files_to(GtkWindow* parent, FmPathList* files, gboolean is_move);
+#define fm_move_files_to(parent, files)   fm_move_or_copy_files_to(parent, files, TRUE)
+#define fm_copy_files_to(parent, files)   fm_move_or_copy_files_to(parent, files, FALSE)
 
-void fm_trash_files(FmPathList* files);
-void fm_delete_files(FmPathList* files);
+void fm_trash_files(GtkWindow* parent, FmPathList* files);
+void fm_delete_files(GtkWindow* parent, FmPathList* files);
 /* trash or delete files according to FmConfig::use_trash. */
-void fm_trash_or_delete_files(FmPathList* files);
+void fm_trash_or_delete_files(GtkWindow* parent, FmPathList* files);
 
-void fm_untrash_files(FmPathList* files);
+void fm_untrash_files(GtkWindow* parent, FmPathList* files);
 
 /* void fm_rename_files(FmPathList* files); */
-void fm_rename_file(FmPath* file);
+void fm_rename_file(GtkWindow* parent, FmPath* file);
 
-void fm_empty_trash();
-
-gboolean fm_launch_files_simple(GtkWindow* parent, GAppLaunchContext* ctx, GList* file_infos, FmLaunchFolderFunc func, gpointer user_data);
-gboolean fm_launch_file_simple(GtkWindow* parent, GAppLaunchContext* ctx, FmFileInfo* file_info, FmLaunchFolderFunc func, gpointer user_data);
-
-gboolean fm_launch_paths_simple(GtkWindow* parent, GAppLaunchContext* ctx, GList* paths, FmLaunchFolderFunc func, gpointer user_data);
-gboolean fm_launch_path_simple(GtkWindow* parent, GAppLaunchContext* ctx, FmPath* path, FmLaunchFolderFunc func, gpointer user_data);
+void fm_empty_trash(GtkWindow* parent);
 
 G_END_DECLS
 
